/*
 * Input-Output on dna (FASTA or PLAIN) text
 */

#ifndef __IODNA_H__
#define __IODNA_H__
//STL
#include <vector>
#include <list>
//STD
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <cmath>
#include <string> // string class
//STR
using namespace std;
#include "macro.h"

/*
 * Structure used to keep DNA sequence
 */


class seqdna {
 public:
  string                name;
  int                   bsize;
  vector<unsigned char> data;
#ifdef MASKED
  vector<int>           maskpos;
#endif

 public:
  /* Get size */
  inline int  size()     {return data.size();};
  inline int  bitsize()  {return bsize;};  
  inline int  dnasize()  {return bsize>>1;}; 
  
  /* Get sequence/subsequence */
  inline unsigned char getQuadNucl(int pos){
    int leftdec  = ((pos & 0x3)<<1);
    int rightdec = 8 - leftdec;
    int realpos  = pos>>2;
    if (leftdec){
      return  ((data[realpos])<<(leftdec)) | ((data[realpos+1])>>(rightdec));
    }else{
      return (data[realpos]);
    }
  }
  
  inline unsigned char getNucl(int pos){
    int leftdec  = 6 - ((pos & 0x3)<<1);    
    int realpos  = pos>>2;
    return  ((data[realpos])>>leftdec) & 0x3;
  }
  
  ostream& display(int pos, int length, ostream& os);
  friend ostream& operator<<(ostream& os, seqdna& seq);
};



/*
 *  Input (Output) on DNA Files  
 */

class iodna
{
 public:  
  iodna();
  iodna(string filename, bool concatseq = false);
  ~iodna();
  
  /* Handling methods */
  int addFile(string filename, bool concatseq = false);
  
  /* Read methods */
  seqdna          &getSequence(int nb){return _sequences[nb];};
  vector<seqdna>  &getSequences()     {return _sequences;};
  unsigned        dnasize();
  unsigned        bitsize();
  unsigned           size();
  /* Parameters */
  void   writeFile(string & filename);
  
  /* IO methods */
  void   display(int i, int j, int length);
  friend ostream& operator<<(ostream& os, iodna& io);
 
 protected:
  vector<seqdna> _sequences;
};

#endif
